package com.lyg.application.controller;

import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyg.application.DTO.ErrorDto;
import com.lyg.application.entity.TaIncompatible;
import com.lyg.application.service.TaIncompatibleService;
import com.lyg.application.util.Rule;
import com.lyg.common.util.StringUtil;
import com.lyg.framework.accesslimit.AccessLimit;
import com.lyg.framework.log.OpLog;
import com.lyg.system.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

/**
 * 不相容;(ta_incompatible)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2024-3-6
 */
@Api(tags = "不相容对象功能接口")
@RestController
@RequestMapping("/")
public class TaIncompatibleController extends BaseController {

    @Autowired
    Rule rule;
    @Autowired
    private TaIncompatibleService taIncompatibleService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @OpLog(module = "不相容", action = "查询单条数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taIncompatible/{id}")
    public SaResult queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return SaResult.data(taIncompatibleService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @OpLog(module = "不相容", action = "分页列表查询")
    @AccessLimit()
    @ApiOperation("分页查询")
    @GetMapping("/taIncompatible")
    public SaResult list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                         @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                         @ApiParam("原岗位") @RequestParam(value = "primaryPost", required = false) String primaryPost,
                         @ApiParam("目标岗位") @RequestParam(value = "targetPost", required = false) String targetPost,
                         @ApiParam("规则") @RequestParam(value = "rule", required = false) String rule,

                         @ApiParam("正序排列") @RequestParam(value = "sortAsc", required = false) String sortAsc,
                         @ApiParam("倒序排列") @RequestParam(value = "sortDesc", defaultValue = "create_date") String sortDesc) throws Exception {


        IPage<TaIncompatible> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaIncompatible> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtil.isNotEmpty(primaryPost), "primary_post", primaryPost);
        queryWrapper.eq(StringUtil.isNotEmpty(targetPost), "target_post", targetPost);
        queryWrapper.eq(StringUtil.isNotEmpty(rule), "rule", rule);
        queryWrapper.orderByAsc(StringUtil.isNotEmpty(sortAsc), StringUtil.humpToLine(sortAsc));
        queryWrapper.orderByDesc(StringUtil.isNotEmpty(sortDesc), StringUtil.humpToLine(sortDesc));
        IPage<TaIncompatible> result = taIncompatibleService.page(pg, queryWrapper);

        return SaResult.data(result);
    }

    /**
     * 新增数据
     *
     * @param taIncompatible 实例对象
     * @return 实例对象
     */
    @OpLog(module = "不相容", action = "新增数据")
    @AccessLimit()
    @ApiOperation("新增数据")
    @PostMapping("/taIncompatible")
    public SaResult add(@ApiParam("对象实体") @Validated @RequestBody TaIncompatible taIncompatible) throws Exception {
        taIncompatibleService.save(taIncompatible);
        return SaResult.data(taIncompatible);
    }

    /**
     * 更新数据
     *
     * @param taIncompatible 实例对象
     * @return 实例对象
     */
    @OpLog(module = "不相容", action = "更新数据")
    @AccessLimit()
    @ApiOperation("更新数据")
    @PutMapping("/taIncompatible/{id}")
    public SaResult edit(@ApiParam("对象实体") @Validated @RequestBody TaIncompatible taIncompatible,
                         @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taIncompatible.setIncompatibleId(id);
        taIncompatibleService.updateById(taIncompatible);
        return SaResult.data(taIncompatible);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @OpLog(module = "不相容", action = "删除数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taIncompatible/{id}")
    public SaResult deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taIncompatibleService.removeById(id);
        return SaResult.data("success");
    }


    /**
     *
     */
    @OpLog(module = "不相容规则查询", action = "")
    @AccessLimit()
    @ApiOperation("不相容规则查询")
    @GetMapping("/taIncompatible/rule")
    public SaResult listRule(
            @ApiParam("当前时间") @RequestParam(value = "passTime", required = false) String passTime,
            @ApiParam("原机构/网点") @RequestParam(value = "oldOrgCode", required = false) String oldOrgCode,
            @ApiParam("原岗位") @RequestParam(value = "primaryPost", required = false) String primaryPost,
            @ApiParam("目标机构/网点") @RequestParam(value = "newOrgCode", required = false) String newOrgCode,
            @ApiParam("目标岗位") @RequestParam(value = "targetPost", required = false) String targetPost) throws Exception {
        Map<String, String> params = new Hashtable<>();

        if (oldOrgCode != null && newOrgCode != null && primaryPost != null && targetPost != null && passTime != null) {
            params.put("oldOrgCode", oldOrgCode);
            params.put("newOrgCode", newOrgCode);
            params.put("primaryPost", primaryPost);
            params.put("targetPost", targetPost);
            params.put("passTime", passTime);

            List<ErrorDto> result = rule.siftRule(params);

            return SaResult.data(result);

        }
        return SaResult.error("请输入岗位或机构号");


    }


}