package com.lyg.application.controller;

import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyg.application.entity.TaMandatoryLeave;
import com.lyg.application.service.TaMandatoryLeaveService;
import com.lyg.common.util.StringUtil;
import com.lyg.framework.accesslimit.AccessLimit;
import com.lyg.framework.log.OpLog;
import com.lyg.system.controller.BaseController;
import com.lyg.system.mapper.SysOrgMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 强制休假表;(ta_mandatory_leave)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2024-3-6
 */
@Api(tags = "强制休假表对象功能接口")
@RestController
@RequestMapping("/")
public class TaMandatoryLeaveController extends BaseController {

    @Autowired
    private TaMandatoryLeaveService taMandatoryLeaveService;
    @Autowired
    private SysOrgMapper sysOrgMapper;


    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @OpLog(module = "强制休假表", action = "查询单条数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taMandatoryLeave/{id}")
    public SaResult queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaMandatoryLeave taMandatoryLeave = taMandatoryLeaveService.getById(id);
        Boolean b = checkAuth() == null || checkAuth() == taMandatoryLeave.getCreateUser();

        if (b) {
            return SaResult.data(taMandatoryLeaveService.getById(id));
        }
        return SaResult.error("无权限");
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @OpLog(module = "强制休假表", action = "分页列表查询")
    @AccessLimit()
    @ApiOperation("分页查询")
    @GetMapping("/taMandatoryLeave")
    public SaResult list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                         @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                         @ApiParam("总行部门/一级分行") @RequestParam(value = "headOfficeOrg", required = false) String headOfficeOrg,
                         @ApiParam("员工工号") @RequestParam(value = "employeeNum", required = false) String employeeNum,
                         @ApiParam("员工姓名") @RequestParam(value = "employeeName", required = false) String employeeName,
                         @ApiParam("所在单位机构号") @RequestParam(value = "orgNum", required = false) String orgNum,
                         @ApiParam("所在单位名称") @RequestParam(value = "orgName", required = false) String orgName,
                         @ApiParam("对应机构层次") @RequestParam(value = "counterInstitutionsLevel", required = false) String counterInstitutionsLevel,
                         @ApiParam("对应重要岗位") @RequestParam(value = "counterImpPost", required = false) String counterImpPost,
                         @ApiParam("对应总行条线") @RequestParam(value = "counterHeadOfficeLine", required = false) String counterHeadOfficeLine,

                         @ApiParam("休假起始时间开始") @RequestParam(value = "startVacation", required = false) String startVacation,
                         @ApiParam("休假起始时间结束") @RequestParam(value = "endVacation", required = false) String endVacation,


                         @ApiParam("正序排列") @RequestParam(value = "sortAsc", defaultValue = "create_date") String sortAsc,
                         @ApiParam("倒序排列") @RequestParam(value = "sortDesc", required = false) String sortDesc
    ) throws Exception {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 定义日期格式化模式
        LocalDate startDate = null;
        LocalDate endDate = null;
        if (StringUtil.isNotEmpty(startVacation)) {
            startDate = LocalDate.parse(startVacation, formatter);
            endDate = LocalDate.parse(endVacation, formatter);
        }
        IPage<TaMandatoryLeave> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaMandatoryLeave> queryWrapper = new QueryWrapper<>();

        queryWrapper.between(startDate != null, "start_time_vacation", startDate, endDate);

        queryWrapper.eq(StringUtil.isNotEmpty(checkAuth()), "CREATE_USER", checkAuth());
        queryWrapper.like(StringUtil.isNotEmpty(headOfficeOrg), "head_office_org", headOfficeOrg);
        queryWrapper.eq(StringUtil.isNotEmpty(employeeNum), "employee_num", employeeNum);
        queryWrapper.like(StringUtil.isNotEmpty(employeeName), "employee_name", employeeName);
        queryWrapper.eq(StringUtil.isNotEmpty(orgNum), "org_num", orgNum);
        queryWrapper.like(StringUtil.isNotEmpty(orgName), "org_name", orgName);
        queryWrapper.like(StringUtil.isNotEmpty(counterInstitutionsLevel), "counter_institutions_level", counterInstitutionsLevel);
        queryWrapper.like(StringUtil.isNotEmpty(counterImpPost), "counter_imp_post", counterImpPost);
        queryWrapper.like(StringUtil.isNotEmpty(counterHeadOfficeLine), "counter_head_office_line", counterHeadOfficeLine);
        queryWrapper.orderByAsc(StringUtil.isNotEmpty(sortAsc), StringUtil.humpToLine(sortAsc));
        queryWrapper.orderByDesc(StringUtil.isNotEmpty(sortDesc), StringUtil.humpToLine(sortDesc));
        IPage<TaMandatoryLeave> result = taMandatoryLeaveService.page(pg, queryWrapper);

        return SaResult.data(result);
    }

    /**
     * 新增数据
     *
     * @param taMandatoryLeave 实例对象
     * @return 实例对象
     */
    @OpLog(module = "强制休假表", action = "新增数据")
    @AccessLimit()
    @ApiOperation("新增数据")
    @PostMapping("/taMandatoryLeave")
    public SaResult add(@ApiParam("对象实体") @Validated @RequestBody TaMandatoryLeave taMandatoryLeave) throws Exception {
        taMandatoryLeaveService.save(taMandatoryLeave);
        return SaResult.data(taMandatoryLeave);
    }

    /**
     * 更新数据
     *
     * @param taMandatoryLeave 实例对象
     * @return 实例对象
     */
    @OpLog(module = "强制休假表", action = "更新数据")
    @AccessLimit()
    @ApiOperation("更新数据")
    @PutMapping("/taMandatoryLeave/{id}")
    public SaResult edit(@ApiParam("对象实体") @Validated @RequestBody TaMandatoryLeave taMandatoryLeave,
                         @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taMandatoryLeaveService.updateById(taMandatoryLeave);
        return SaResult.data(taMandatoryLeave);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @OpLog(module = "强制休假表", action = "删除数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taMandatoryLeave/{id}")
    public SaResult deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taMandatoryLeaveService.removeById(id);
        return SaResult.data("success");
    }
}