package com.lyg.application.controller;

import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyg.application.entity.TaRotation;
import com.lyg.application.service.TaRotationService;
import com.lyg.application.util.Rule;
import com.lyg.common.util.StringUtil;
import com.lyg.framework.accesslimit.AccessLimit;
import com.lyg.framework.log.OpLog;
import com.lyg.system.controller.BaseController;
import com.lyg.system.mapper.SysOrgMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 轮岗列表;(ta_rotation)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2024-3-6
 */
@Api(tags = "轮岗列表对象功能接口")
@RestController
@RequestMapping("/")
public class TaRotationController extends BaseController {

    @Autowired
    private TaRotationService taRotationService;
    @Autowired
    private Rule rule;

    @Autowired
    private SysOrgMapper sysOrgMapper;


    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @OpLog(module = "轮岗列表", action = "查询单条数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taRotation/{id}")
    public SaResult queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {

        TaRotation taRotation = taRotationService.getById(id);

        Boolean b = checkAuth() == null || checkAuth() == taRotation.getCreateUser();
        if (b) {
            return SaResult.data(taRotationService.getById(id));
        }
        return SaResult.error("无权限");

    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @OpLog(module = "轮岗列表", action = "分页列表查询")
    @AccessLimit()
    @ApiOperation("分页查询")
    @GetMapping("/taRotation")
    public SaResult list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                         @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                         @ApiParam("总行部门/一级分行") @RequestParam(value = "headOfficeOrg", required = false) String headOfficeOrg,
                         @ApiParam("员工工号") @RequestParam(value = "employeeNum", required = false) String employeeNum,
                         @ApiParam("员工姓名") @RequestParam(value = "employeeName", required = false) String employeeName,
                         @ApiParam("具体单位机构号") @RequestParam(value = "specificOrgNum", required = false) String specificOrgNum,
                         @ApiParam("所在单位具体名称") @RequestParam(value = "specificOrgName", required = false) String specificOrgName,
                         @ApiParam("对应机构层次") @RequestParam(value = "counterInstitutionsLevel", required = false) String counterInstitutionsLevel,
                         @ApiParam("对应重要岗位") @RequestParam(value = "counterImpPositions", required = false) String counterImpPositions,
                         @ApiParam("对应总行条线") @RequestParam(value = "counterHeadOfficeLine", required = false) String counterHeadOfficeLine,
                         @ApiParam("轮岗截止日期开始") @RequestParam(value = "startDeadline", required = false) String startDeadline,
                         @ApiParam("轮岗截止日期结束") @RequestParam(value = "endDeadline", required = false) String endDeadline,

                         @ApiParam("轮岗执行时间开始") @RequestParam(value = "startExeTime", required = false) String startExeTime,
                         @ApiParam("轮岗执行时间结束") @RequestParam(value = "endExeTime", required = false) String endExeTime,

                         @ApiParam("正序排列") @RequestParam(value = "sortAsc", defaultValue = "create_date") String sortAsc,
                         @ApiParam("倒序排列") @RequestParam(value = "sortDesc", required = false) String sortDesc
    ) throws Exception {

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 定义日期格式化模式
        LocalDate startDate = null;
        LocalDate endDate = null;
        if (StringUtil.isNotEmpty(startDeadline)) {
            startDate = LocalDate.parse(startDeadline, formatter);
            endDate = LocalDate.parse(endDeadline, formatter);
        }
        if (StringUtil.isNotEmpty(startExeTime)) {
            startDate = LocalDate.parse(startExeTime, formatter);
            endDate = LocalDate.parse(endExeTime, formatter);
        }


        IPage<TaRotation> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaRotation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtil.isNotEmpty(checkAuth()), "CREATE_USER", checkAuth());
        queryWrapper.between(startDate != null, "rotation_deadline", startDate, endDate);
        queryWrapper.between(startDate != null, "rotation_exe_time", startDate, endDate);


        queryWrapper.like(StringUtil.isNotEmpty(headOfficeOrg), "head_office_org", headOfficeOrg);
        queryWrapper.eq(StringUtil.isNotEmpty(employeeNum), "employee_num", employeeNum);
        queryWrapper.like(StringUtil.isNotEmpty(employeeName), "employee_name", employeeName);
        queryWrapper.eq(StringUtil.isNotEmpty(specificOrgNum), "specific_org_num", specificOrgNum);
        queryWrapper.like(StringUtil.isNotEmpty(specificOrgName), "specific_org_name", specificOrgName);
        queryWrapper.like(StringUtil.isNotEmpty(counterInstitutionsLevel), "counter_institutions_level", counterInstitutionsLevel);
        queryWrapper.like(StringUtil.isNotEmpty(counterImpPositions), "counter_imp_positions", counterImpPositions);
        queryWrapper.like(StringUtil.isNotEmpty(counterHeadOfficeLine), "counter_head_office_line", counterHeadOfficeLine);

        queryWrapper.orderByAsc(StringUtil.isNotEmpty(sortAsc), StringUtil.humpToLine(sortAsc));
        queryWrapper.orderByDesc(StringUtil.isNotEmpty(sortDesc), StringUtil.humpToLine(sortDesc));
        IPage<TaRotation> result = taRotationService.page(pg, queryWrapper);

        return SaResult.data(result);
    }

    /**
     * 新增数据
     *
     * @param taRotation 实例对象
     * @return 实例对象
     */
    @OpLog(module = "轮岗列表", action = "新增数据")
    @AccessLimit()
    @ApiOperation("新增数据")
    @PostMapping("/taRotation")
    public SaResult add(@ApiParam("对象实体") @Validated @RequestBody TaRotation taRotation) throws Exception {
        taRotationService.save(taRotation);
        Object result = rule.taRotationRule(taRotation);

//        LocalDate localDate = LocalDate.now();
//        DateTimeFormatter ft = DateTimeFormatter.ofPattern("yyyy-MM-dd");
//        String oldOrgCode = taRotation.getSpecificOrgNum();
//        String newOrgCode = taRotation.getOrgNumJobRotation();
//        String primaryPost = taRotation.getCounterImpPositions();
//        String targetPost = taRotation.getCounterImpPostRotation();
//        String passTime = localDate.format(ft);
//        if (oldOrgCode != null && newOrgCode != null && primaryPost != null && targetPost != null && passTime != null) {
//
//            Map<String, String> params = new HashMap<>();
//            params.put("oldOrgCode", oldOrgCode);
//            params.put("newOrgCode", newOrgCode);
//            params.put("primaryPost", primaryPost);
//            params.put("targetPost", targetPost);
//            params.put("passTime", passTime);
//
//            List<ErrorDto> result = rule.siftRule(params);
//        }
        return (SaResult) result;
    }

    /**
     * 更新数据
     *
     * @param taRotation 实例对象
     * @return 实例对象
     */
    @OpLog(module = "轮岗列表", action = "更新数据")
    @AccessLimit()
    @ApiOperation("更新数据")
    @PutMapping("/taRotation/{id}")
    public SaResult edit(@ApiParam("对象实体") @Validated @RequestBody TaRotation taRotation,
                         @ApiParam("对象ID") @PathVariable String id) throws Exception {

        taRotation.setRotationId(id);
        taRotationService.updateById(taRotation);
        Object result = rule.taRotationRule(taRotation);
//        boolean isAllCompatible = true;
//        if (result instanceof List) {
//            ArrayList<ErrorDto> resultList = (ArrayList<ErrorDto>) rule.taRotationRule(taRotation);
//            for (ErrorDto errorDto : resultList) {
//                String regularRuleMessage = errorDto.getRegularRules();
//                String monthRules = errorDto.getMonthRules();
//                if (regularRuleMessage != null||monthRules != null) {
//                    // 判断并添加相容/不相容信息到消息中
//                    if (regularRuleMessage.contains("是不相容的")|| monthRules.contains("是不相容的")) {
//                        isAllCompatible = true;
//                    } else if (regularRuleMessage.contains("是相容的")|| monthRules.contains("是相容的")) {
//                        isAllCompatible = false;
//                    }
//                }
//            }
//        } else if (result instanceof TaRotation) {
//
//        }


// 设置规则的 id（假设 You

        return (SaResult) result;

    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @OpLog(module = "轮岗列表", action = "删除数据", desc = "ID：{{#id}}")
    @AccessLimit()
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taRotation/{id}")
    public SaResult deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taRotationService.removeById(id);
        return SaResult.data("success");
    }
}