package com.lyg.application.controller;

import cn.dev33.satoken.util.SaResult;
import com.alibaba.excel.EasyExcel;
import com.lyg.application.Listener.TaMandatoryLeaveListener;
import com.lyg.application.Listener.TaRotationListener;
import com.lyg.application.entity.TaMandatoryLeave;
import com.lyg.application.entity.TaRotation;
import com.lyg.application.service.TaMandatoryLeaveService;
import com.lyg.application.service.TaRotationService;
import com.lyg.application.service.impl.TaMandatoryLeaveServiceImpl;
import com.lyg.application.service.impl.TaRotationServiceImpl;
import com.lyg.framework.accesslimit.AccessLimit;
import com.lyg.system.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;


/**
 * 客户生日维护经理表;(客户生日维护经理表)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2024-1-29
 */
@Api(tags = "上传")
@RestController
@RequestMapping("/")
@ResponseBody
public class UploadController extends BaseController {


    @Value("${yz.warningTmp}")
    String warning;
    @Value("${yz.promptTmp}")
    String prompt;
    @Autowired
    private TaRotationService taRotationService;
    @Autowired
    private TaRotationServiceImpl taRotationServiceImpl;
    @Autowired
    private TaMandatoryLeaveService taMandatoryLeaveService;
    @Autowired
    private TaMandatoryLeaveServiceImpl taMandatoryLeaveServiceImpl;

    @AccessLimit()
    @ApiOperation("上传文件")
    @PostMapping("/upload/taRotation")
    public SaResult upload(@ApiParam("文件") @RequestParam("file") MultipartFile file) throws IOException {
        if (file.isEmpty()) {//检查文件是否为空
            throw new IllegalArgumentException("文件不能为空");

        }
        try {

            InputStream inputStream = file.getInputStream();
            TaRotationListener taRotationListener = new TaRotationListener(taRotationService);
            EasyExcel.read(inputStream, TaRotation.class, taRotationListener).sheet().headRowNumber(4).doRead();

            List<Object> errorMessages = taRotationServiceImpl.getErrorMessages();
            StringBuilder errorMessageBuilder = new StringBuilder();

            for (Object errorMessage : errorMessages) {
                errorMessageBuilder.append(errorMessage).append("\n");
            }

            errorMessages.clear();
            return SaResult.data("读取文件失败:" + errorMessageBuilder);


        } catch (Exception e) {
            e.printStackTrace();
            return SaResult.error("读取文件失败: " + e.getMessage());
        }

    }


    @AccessLimit()
    @ApiOperation("上传文件")
    @PostMapping("/upload/taMandatoryLeave")
    public SaResult uploadLeave(@ApiParam("文件") @RequestParam("file") MultipartFile file) throws IOException {
        if (file.isEmpty()) {//检查文件是否为空
            throw new IllegalArgumentException("文件不能为空");

        }
        try {

            InputStream inputStream = file.getInputStream();
            TaMandatoryLeaveListener taMandatoryLeaveListener = new TaMandatoryLeaveListener(taMandatoryLeaveService);
            EasyExcel.read(inputStream, TaMandatoryLeave.class, taMandatoryLeaveListener).sheet().headRowNumber(4).doRead();

            List<String> errorMessages = taMandatoryLeaveServiceImpl.getErrorMessages();
            StringBuilder errorMessageBuilder = new StringBuilder();

            for (String errorMessage : errorMessages) {
                errorMessageBuilder.append(errorMessage).append("\n");
            }

            errorMessages.clear();
            return SaResult.data("读取文件失败:" + errorMessageBuilder);


        } catch (Exception e) {
            e.printStackTrace();
            return SaResult.error("读取文件失败: " + e.getMessage());
        }

    }



    @AccessLimit()
//    @Scheduled(cron = "0 0 8 * * *")
    @GetMapping("/as/holiday")
    public void promptWarning() throws Exception {

//        taRotationService.selectGetWaring();
        taMandatoryLeaveService.selectGetPrompt();

    }

}
