package com.lyg.application.util;

import com.lyg.application.entity.TaMandatoryLeave;
import com.lyg.application.entity.TaMessage;
import com.lyg.application.entity.TaRotation;
import com.lyg.application.entity.TaSending;
import com.lyg.application.mapper.TaMandatoryLeaveMapper;
import com.lyg.application.mapper.TaMessageMapper;
import com.lyg.application.mapper.TaSendingMapper;
import com.lyg.application.service.TaMessageService;
import com.lyg.common.Constants;
import com.lyg.common.util.StringUtil;
import com.lyg.system.entity.SysUser;
import com.lyg.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

@Component
public class MessageHandle {

    private static final String MESSAGE_SOURCE_NAME = "taRotation";


    @Autowired
    private TaMessageService taMessageService;
    @Autowired
    private TaMessageMapper taMessageMapper;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private TaSendingMapper taSendingMapper;

    @Autowired
    private TaMandatoryLeaveMapper taMandatoryLeaveMapper;

    /**
     * 处理消息的通用逻辑
     *
     * @param localDate        日期
     * @param messageType      消息类型
     * @param dataFetchingFunc 数据获取的函数
     */
    public void handleMessages(LocalDate localDate, String tmp, String messageType, String sourceName, Function<LocalDate, List<TaRotation>> dataFetchingFunc) {
        List<TaRotation> messageList = dataFetchingFunc.apply(localDate);

        if (!messageList.isEmpty()) {
            // 在循环外初始化StringBuilder

            for (TaRotation item : messageList) {
                processItem(item, tmp, messageType, sourceName);
            }
        }
    }


    /**
     * 处理每个项目的消息逻辑
     *
     * @param item        数据项
     * @param messageType 消息类型
     */
    private void processItem(TaRotation item, String tmp, String messageType, String sourceName) {
        String newTmp = null;
        TaMessage taMessage = new TaMessage();
        Map<String, String> params = new HashMap<>();


        String uuid = StringUtil.UUID();
        taMessage.setMessageId(uuid);
        taMessage.setSourceId(item.getRotationId());

//        // 使用描述性强的常量代替硬编码值
//        stringBuilder.insert(Constants.numI, item.getEmployeeNum());
//        int insertPos = item.getEmployeeNum().length() + Constants.nameI;
//        stringBuilder.insert(insertPos, item.getEmployeeName());
        String employeeNum = item.getEmployeeNum();
        String employeeName = item.getEmployeeName();
        newTmp = tmp.replace("code", employeeNum).replace("name", employeeName);
        taMessage.setContentSent(newTmp);
        taMessage.setSourceName(sourceName);
        taMessage.setWaringType(messageType);
        params.put("sourceName", sourceName);
        params.put("waringType", messageType);
        params.put("contentSent", newTmp);
        Long count = taMessageService.countBy("source_id", item.getRotationId(), true);
        if (count == 0) {
            taMessageMapper.insert(taMessage);
            insertIntoTaSending(item.getCreateUser(), uuid);
        } else {
            taMessageMapper.updateBySourceId(item.getRotationId(), params);
        }
    }


    public void handleMandatoryLeaves(LocalDate localDate, String tmp, String messageType, String sourceName, Function<LocalDate, List<TaMandatoryLeave>> dataFetchingFunc) {
        List<TaMandatoryLeave> messageList = dataFetchingFunc.apply(localDate);

        if (!messageList.isEmpty()) {

            for (TaMandatoryLeave item : messageList) {
                processItemLeaves(item, tmp, messageType, sourceName);
            }
        }
    }

    private void processItemLeaves(TaMandatoryLeave item, String tmp, String messageType, String sourceName) {
        TaMessage taMessage = new TaMessage();
        Map<String, String> params = new HashMap<>();

        String uuid = StringUtil.UUID();
        taMessage.setMessageId(uuid);
        taMessage.setSourceId(item.getLeaveId());

//        // 使用描述性强的常量代替硬编码值
        StringBuilder sb = new StringBuilder(tmp);
        // 假设warningTmp或promptTmp已经定义且包含插入点
//        stringBuilder.insert(Constants.numI, item.getEmployeeNum());
//        int insertPos = item.getEmployeeNum().length() + Constants.nameI;
//        stringBuilder.insert(insertPos, item.getEmployeeName());
        String employeeNum = item.getEmployeeNum();
        String employeeName = item.getEmployeeName();
        String newTmp = null;
        newTmp = tmp.replace("code", employeeNum).replace("name", employeeName);
        taMessage.setContentSent(newTmp);
        taMessage.setSourceName(sourceName);
        taMessage.setWaringType(messageType);
        params.put("sourceName", "taMandatoryLeave");
        params.put("waringType", "prompt");
        params.put("contentSent", newTmp);
        Long count = taMessageService.countBy("source_id", item.getLeaveId(), true);
        if (count == 0) {
            taMessageMapper.insert(taMessage);
            insertIntoTaSending(item.getCreateUser(), uuid);
        } else {
            taMessageMapper.updateBySourceId(item.getLeaveId(), params);
        }
    }


    public void insertIntoTaSending(String createUser, String uuid) {

        //                item.getCreateUser()  就是新增这个数据的人
        //现在需要去重 创建人和管理员  是一个人只能往  Tasending插入一条数据否则是两条数据
        SysUser sysUserAdmin = sysUserMapper.getAdmin(createUser);
//                如果为空 就不是管理员创建的;
        List<String> userIdList = new ArrayList<>();

        if (sysUserAdmin == null) {
            userIdList.add(createUser);
        }
        userIdList = sysUserMapper.allAdmin();

        for (String it : userIdList) {
            TaSending taSending = new TaSending();
            taSending.setMessageId(uuid);
            taSending.setRecipientId(it);
            taSendingMapper.insert(taSending);
        }

    }


}
