package com.lyg.framework.mybatisplus;

import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.handler.TenantLineHandler;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.TenantLineInnerInterceptor;
import com.lyg.common.util.MyPropertySourceFactory;
import com.lyg.framework.config.TenantConfig;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.StringValue;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

//@PropertySource(value = {"framework.yml"}, factory = MyPropertySourceFactory.class)
@Configuration
public class MybatisPlusConfig {
    @Resource
    HttpServletRequest request;

    @Resource
    TenantConfig tenantConfig;

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();

        // 分页插件
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.ORACLE));

        if (tenantConfig.getEnabled()) {
            // 多租户插件
            interceptor.addInnerInterceptor(new TenantLineInnerInterceptor(new TenantLineHandler() {
                @Override
                public Expression getTenantId() {
                    // request 中的 tenantId 是在 sa-token 中设置的
                    String tenantId = (String) request.getAttribute("tenantId");
                    return new StringValue(tenantId == null ? tenantConfig.getDefaultId() : tenantId);
                }

                @Override
                public String getTenantIdColumn() {
                    return "tenant_id";
                }

                // 设置不用添加租户查询条件的表
                @Override
                public boolean ignoreTable(String tableName) {
                    return tenantConfig.getWhiteList().contains(tableName);
                }
            }));
        }

        return interceptor;
    }




}
