package com.lyg.framework.satoken;

import cn.dev33.satoken.context.SaHolder;
import cn.dev33.satoken.exception.SaTokenException;
import cn.dev33.satoken.filter.SaServletFilter;
import cn.dev33.satoken.router.SaHttpMethod;
import cn.dev33.satoken.router.SaRouter;
import cn.dev33.satoken.stp.StpUtil;
import cn.dev33.satoken.util.SaResult;
import com.lyg.framework.config.FilterConfig;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;

@Configuration
public class SaTokenConfigure {

    @Resource
    FilterConfig filterConfig;

    @Bean
    public SaServletFilter getSaServletFilter() {
        SaServletFilter saServletFilter = new SaServletFilter();
        saServletFilter.addInclude("/**");
        saServletFilter.addExclude(filterConfig.getAnnList().toArray(new String[0]));
        saServletFilter.setAuth(auth -> {
            // OPTIONS 请求不校验, 主要是为了解决跨域问题
            SaRouter.notMatch(SaHttpMethod.OPTIONS).check(r -> {
                // 平台账户或者租户必须登录一个
                if (!StpUtil.isLogin()) {
                    throw new SaTokenException("请登录后再访问接口");
                }
            });
        });

        saServletFilter.setError(err -> {
            SaHolder.getResponse().setHeader("Content-Type", "application/json;charset=UTF-8");
            SaHolder.getResponse().setStatus(200);
            return SaResult.error(err.getMessage()).setCode(401);
        });

        return saServletFilter;
    }

    @Bean
    public SaServletFilter2 getSaServletFilter2() {
        return new SaServletFilter2();
    }
}